/*
 *  Copyright 2014, Enguerrand de Rochefort
 * 
 * This file is part of xdat.
 *
 * xdat is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * xdat is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xdat.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.xdat.actionListeners.parallelCoordinatesChartFrame;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.xdat.Main;
import org.xdat.chart.ParallelChart;
import org.xdat.chart.Axis;
import org.xdat.chart.ParallelCoordinatesChart;
import org.xdat.chart.ParallelSetsChart;
import org.xdat.data.DataSheet;
import org.xdat.data.Design;
import org.xdat.gui.frames.ChartFrame;
import org.xdat.gui.panels.ParallelCoordinatesChartPanel;
import org.xdat.gui.tables.DataSheetTableColumnModel;

/**
 * ActionListener that is used for the context menu on the
 * {@link ParallelCoordinatesChartPanel}.
 * 
 */
public class ParallelChartContextMenuActionListener implements ActionListener {

	/** Flag to enable debug message printing for this class. */
	static final boolean printLog = false;

	/** The main Window. */
	private Main mainWindow;

	/** The chart Frame. */
	private ChartFrame chartFrame;

	/**
	 * The Axis currently being edited.
	 */
	private Axis axis;
        
        private ParallelChart chart;

	/**
	 * Instantiates a new context menu for an axis on a parallel coordinates
	 * chart.
	 * 
	 * @param mainWindow
	 *            the main window
	 * @param chartFrame
	 *            the chart frame
	 * @param axis
	 *            the axis
	 */
	public ParallelChartContextMenuActionListener(Main mainWindow, ChartFrame chartFrame, Axis axis) {
		this.mainWindow = mainWindow;
		this.chartFrame = chartFrame;
		this.axis = axis;
                this.chart = (ParallelChart) chartFrame.getChart();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (actionCommand.equals("moveAxisLeft")) {
                        DataSheetTableColumnModel cModel = (DataSheetTableColumnModel) this.mainWindow.getDataSheetTablePanel().getDataTable().getColumnModel();
                        chart.moveAxisLeft(axis, cModel);
                        chartFrame.getChartPanel().updateLayout();
                        chartFrame.repaint();
		} else if (actionCommand.equals("moveAxisRight")) {
                        DataSheetTableColumnModel cModel = (DataSheetTableColumnModel) this.mainWindow.getDataSheetTablePanel().getDataTable().getColumnModel();
                        chart.moveAxisRight(axis, cModel);
                        chartFrame.getChartPanel().updateLayout();
                        chartFrame.repaint();
		} else if (actionCommand.equals("hideAxis")) {
                        ParallelChart chart = (ParallelChart) chartFrame.getChart();
                        chart.hideAxis(axis);
                        chartFrame.getChartPanel().updateLayout();
			chartFrame.repaint();
		} else if (actionCommand.equals("reduceDistanceThisAxis")) {
			this.axis.setWidth(Math.max(0, axis.getWidth() - 10));
			this.chartFrame.repaint();
		} else if (actionCommand.equals("increaseDistanceThisAxis")) {
			this.axis.setWidth(Math.max(0, axis.getWidth() + 10));
			this.chartFrame.repaint();
		} else if (actionCommand.equals("reduceDistanceAllAxes")) {
			ParallelChart chart = this.axis.getChart();
			for (int i = 0; i < chart.getAxisCount(); i++) {
				chart.getAxis(i).setWidth(Math.max(0, chart.getAxis(i).getWidth() - 10));
			}
			this.chartFrame.repaint();
		} else if (actionCommand.equals("increaseDistanceAllAxes")) {
			ParallelChart chart = this.axis.getChart();
			for (int i = 0; i < chart.getAxisCount(); i++) {
				chart.getAxis(i).setWidth(Math.max(0, chart.getAxis(i).getWidth() + 10));
			}
			this.chartFrame.repaint();
		} else if (actionCommand.equals("invertAxis")) {
			this.axis.setAxisInverted(!this.axis.isAxisInverted());
			this.chartFrame.repaint();		
		} else {
			System.out.println(e.getActionCommand());
		}
	}

	/**
	 * Prints debug information to stdout when printLog is set to true.
	 * 
	 * @param message
	 *            the message
	 */
	private void log(String message) {
		if (ParallelChartContextMenuActionListener.printLog && Main.isLoggingEnabled()) {
			System.out.println(this.getClass().getName() + "." + message);
		}
	}

}
