/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.xdat.chart;

import java.util.ArrayList;
import java.util.Locale;
import org.xdat.data.Design;
import org.xdat.data.Parameter;

/**
 *
 * @author cmolikl
 */
public class AxisCategoryRangeCondition extends AxisCategoryConditionAbstract {
    
    private double min;
    private double max;
    private boolean closedEndRange;
    private Parameter p;
    private ParallelSetsAxis axis;
    
    public AxisCategoryRangeCondition(double min, double max, Parameter p) {
        this(min, max, false, p);
    }
    
    public AxisCategoryRangeCondition(double min, double max, boolean closedEndRange, Parameter p) {
        this.min = min;
        this.max = max;
        this.closedEndRange = closedEndRange;
        this.p = p;
        axis = (ParallelSetsAxis) p.getAxis();
    }

    @Override
    public boolean test(Design d) {
        if(!p.isNumeric()) return false;
        double value = d.getDoubleValue(p);
        if(isClosedEndRange()) {
            return (value >= min && value <= max);
        }
        else {
            return (value >= min && value < max);
        }
    }
    
    public String getLabel() {
        String minLabel = String.format(Locale.US, "%4.3f", min);
        String maxLabel = String.format(Locale.US, "%4.3f", max);
        return "(" + minLabel + "; " + maxLabel + ")";
    }
    
    public double getMin() {
        return min;
    }
    
    public double getMax() {
        return max;
    }
    
    public int size() {
        return 1;
    }

    /**
     * @return the closedEndRange
     */
    public boolean isClosedEndRange() {
        return closedEndRange;
    }
    
    public Parameter getParameter() {
        return p;
    }

    public ParallelSetsAxis getAxis() {
        return axis;
    }

    @Override
    public AxisCategoryCondition clone() {
        AxisCategoryRangeCondition copy = new AxisCategoryRangeCondition(min, max, closedEndRange, p);
        return copy;
    }
}
