
package org.xdat.chart;

import java.awt.Color;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import org.xdat.data.Design;

/**
 * Representation of parallelogram between axes of parallel sets chart.
 *
 * @author Martin Janda
 */
public class DesignSet implements Serializable {
    private float inputHeight;
    private float outputHeight;
    private float inputY;
    private float outputY;
    private int inputIndex;
    private int outputIndex;
    private AxisCategory inputCategory;
    private AxisCategory outputCategory;
    private List<Design> designs;
    private DesignSet parentSet;
    private List<DesignSet> childSets;
    private List<InsideDesignSet> inputInsideSets;
    private List<InsideDesignSet> outputInsideSets;
    
    private float inputTempHeight;
    private float outputTempHeight;
    private int inputTempSize;
    private int outputTempSize;
    
    private float ratio;
    
    private ParallelSetsFilter filter;
    private AxisCategory parentCategory;

    /**
     * Constructor for tree layout child set
     * set has parent set
     * @param inputIndex
     * @param outputIndex
     * @param inputCategory
     * @param outputCategory
     * @param designs
     * @param parentSet 
     */
    public DesignSet(int inputIndex, int outputIndex, AxisCategory inputCategory, AxisCategory outputCategory, List<Design> designs, DesignSet parentSet) {
        this.inputIndex = inputIndex;
        this.outputIndex = outputIndex;
        this.inputCategory = inputCategory;
        this.outputCategory = outputCategory;
        this.inputHeight = 0;
        this.outputHeight = 0;
        this.inputY = 0;
        this.outputY = 0;
        this.designs = designs;
        this.parentSet = parentSet;
        this.childSets = new ArrayList<>();
        this.inputInsideSets = new ArrayList<>();
        this.outputInsideSets = new ArrayList<>();
        this.inputTempHeight = 0;
        this.outputTempHeight = 0;
        this.inputTempSize = 0;
        this.outputTempSize = 0;
    }
    
    /**
     * Constructor for tree layout root set
     * set has parent category
     * @param inputIndex
     * @param outputIndex
     * @param inputCategory
     * @param outputCategory
     * @param designs
     * @param parentCategory 
     */
    public DesignSet(int inputIndex, int outputIndex, AxisCategory inputCategory, AxisCategory outputCategory, List<Design> designs, AxisCategory parentCategory) {
        this.inputIndex = inputIndex;
        this.outputIndex = outputIndex;
        this.inputCategory = inputCategory;
        this.outputCategory = outputCategory;
        this.inputHeight = 0;
        this.outputHeight = 0;
        this.inputY = 0;
        this.outputY = 0;
        this.designs = designs;
        this.childSets = new ArrayList<>();
        this.inputInsideSets = new ArrayList<>();
        this.outputInsideSets = new ArrayList<>();
        this.inputTempHeight = 0;
        this.outputTempHeight = 0;
        this.inputTempSize = 0;
        this.outputTempSize = 0;
        this.parentCategory = parentCategory;
    }
    
    /**
     * Constructor for bundled layout set
     * set has filter
     * @param inputIndex
     * @param outputIndex
     * @param inputCategory
     * @param outputCategory
     * @param designs
     * @param filter 
     */
    public DesignSet(int inputIndex, int outputIndex, AxisCategory inputCategory, AxisCategory outputCategory, List<Design> designs, ParallelSetsFilter filter) {
        this.inputIndex = inputIndex;
        this.outputIndex = outputIndex;
        this.inputCategory = inputCategory;
        this.outputCategory = outputCategory;
        this.inputHeight = 0;
        this.outputHeight = 0;
        this.inputY = 0;
        this.outputY = 0;
        this.designs = designs;
        this.childSets = new ArrayList<>();
        this.inputInsideSets = new ArrayList<>();
        this.outputInsideSets = new ArrayList<>();
        this.inputTempHeight = 0;
        this.outputTempHeight = 0;
        this.inputTempSize = 0;
        this.outputTempSize = 0;
        this.filter = filter;
    }

    public void increaseInputTempHeight(float height){
        inputTempHeight += height;
        inputTempSize++;
        if(inputInsideSets.size() <= inputTempSize){
            inputTempHeight = 0;
            inputTempSize = 0;
        }
    }
    
    public void increaseOutputTempHeight(float height){
        outputTempHeight += height;
        outputTempSize++;
        if(outputInsideSets.size() <= outputTempSize){
            outputTempHeight = 0;
            outputTempSize = 0;
        }
    }
    
    public float getInputTempHeight() {
        return inputTempHeight;
    }

    public void setInputTempHeight(float inputTempHeight) {
        this.inputTempHeight = inputTempHeight;
    }

    public float getOutputTempHeight() {
        return outputTempHeight;
    }

    public void setOutputTempHeight(float outputTempHeight) {
        this.outputTempHeight = outputTempHeight;
    }

    public List<InsideDesignSet> getInputInsideSets() {
        return inputInsideSets;
    }

    public void setInputInsideSets(List<InsideDesignSet> inputInsideSets) {
        this.inputInsideSets = inputInsideSets;
    }

    public List<InsideDesignSet> getOutputInsideSets() {
        return outputInsideSets;
    }

    public void setOutputInsideSets(List<InsideDesignSet> outputInsideSets) {
        this.outputInsideSets = outputInsideSets;
    }

    public ParallelSetsFilter getFilter() {
        return filter;
    }

    public void setFilter(ParallelSetsFilter filter) {
        this.filter = filter;
    }

    public AxisCategory getParentCategory(){
        return parentCategory;
    }

    public Color getColor() {
        if(parentSet != null){
            return parentSet.getColor();
        }else{
            if(filter != null){
                return filter.getColor();
            }else if(parentCategory != null){
                return parentCategory.getColor();
            }else{
                return ParallelSetsChart.NO_FILTER_COLOR;
            }
        }
    }

    public void setParentCategory(AxisCategory parentCategory) {
        this.parentCategory = parentCategory;
    }

    public int getColorIndex() {
        if(filter != null){
            return filter.getFilterID();
        }else if(parentCategory != null){
            return parentCategory.getOrder();
        }else{
            return Integer.MAX_VALUE;
        }
    }
    
    public DesignSet getParentSet() {
        return parentSet;
    }

    public void setParentSet(DesignSet parentSet) {
        this.parentSet = parentSet;
    }

    public List<DesignSet> getChildSets() {
        return childSets;
    }
    
    public void setChildSets(List<DesignSet> childSets) {
        this.childSets = childSets;
    }

    public List<Design> getDesigns() {
        return designs;
    }

    public void setDesigns(List<Design> designs) {
        this.designs = designs;
    }

    public int getInputIndex() {
        return inputIndex;
    }

    public void setInputIndex(int inputIndex) {
        this.inputIndex = inputIndex;
    }

    public int getOutputIndex() {
        return outputIndex;
    }

    public void setOutputIndex(int outputIndex) {
        this.outputIndex = outputIndex;
    }

    public float getInputY() {
        return inputY;
    }

    public void setInputY(float inputY) {
        this.inputY = inputY;
    }

    public float getOutputY() {
        return outputY;
    }

    public void setOutputY(float outputY) {
        this.outputY = outputY;
    }

    public AxisCategory getInputCategory() {
        return inputCategory;
    }

    public void setInputCategory(AxisCategory inputCategory) {
        this.inputCategory = inputCategory;
    }

    public AxisCategory getOutputCategory() {
        return outputCategory;
    }

    public void setOutputCategory(AxisCategory outputCategory) {
        this.outputCategory = outputCategory;
    }

    public float getInputHeight() {
        return inputHeight;
    }

    public void setInputHeight(float inputHeight) {
        this.inputHeight = inputHeight;
    }

    public float getOutputHeight() {
        return outputHeight;
    }

    public void setOutputHeight(float outputHeight) {
        this.outputHeight = outputHeight;
    }
    
    
    
}
