
package org.xdat.chart;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import org.xdat.data.Design;

/**
 * Representation of designs filter in parallel sets chart.
 * Filter to select and highlight some designs according to filters conditions.
 *
 * @author Martin Janda
 */
public class ParallelSetsFilter extends AxisCategoryAndCondition {
    
    private List<AxisCategory> categories;
    private Color color;
    private boolean active;
    private int filterID;
    protected ArrayList<ParallelSetsAxis> axes = new ArrayList<>();
    
    public ParallelSetsFilter(int filterID) {
        super();
        this.categories = new ArrayList<>();
        this.active = true;
        this.filterID = filterID;
    }
    
    @Override
    public boolean test(Design d) {
        if(conditions.isEmpty()) {
            return true;
        }
        else {
            for(AxisCategoryCondition c : conditions) {
                if(!c.test(d)) return false;
            }
            return true;
        }
    }

    AxisCategoryCondition get(ParallelSetsAxis a) {
        int aIndex = axes.indexOf(a);
        if(aIndex < 0) return null;
        return conditions.get(aIndex);
    }

    AxisCategoryCondition remove(ParallelSetsAxis a) {
        int aIndex = axes.indexOf(a);
        if(aIndex < 0) return null;
        axes.remove(aIndex);
        return conditions.remove(aIndex);
    }

    public int getFilterID() {
        return filterID;
    }

    public void setFilterID(int filterID) {
        this.filterID = filterID;
    }
    
    public AxisCategoryCondition getCondition() {
        return this;
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
    }
    
    public List<AxisCategory> getCategories() {
        return categories;
    }
    
    public void addCategory(AxisCategory category){
        ParallelSetsAxis a = category.getAxis();
        AxisCategoryCondition condition = category.getCondition();
        AxisCategoryOrCondition orFilter = (AxisCategoryOrCondition) get(a);
        if(orFilter == null) {
            orFilter = new AxisCategoryOrCondition(condition);
            addCondition(orFilter);
            axes.add(a);
        }
        else {
            orFilter.addCondition(condition);
        }
        
        categories.add(category);
    }
    
    public boolean contains(AxisCategory category){
        return categories.contains(category);
    }
    
    public void removeCategory(AxisCategory category){
        ParallelSetsAxis a = category.getAxis();
        AxisCategoryOrCondition orFilter = (AxisCategoryOrCondition) get(a);
        if(orFilter != null) {
            orFilter.removeCondition(category.getCondition());
            if(orFilter.size() == 0) {
                remove(a);
            }
        }
        
        categories.remove(category);
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        this.color = color;
    }
    
    public void delete(){
        for (AxisCategory category : categories) {
            category.setSelected(false);
        }
    }
    
    public String getName() {
        return getLabel();
    }
    
//    public void filterData() {
//        //Find parameter with smalest number of designs
//        HashMap<Parameter, Integer> designsCountMap = new HashMap<>();
//        for(AxisCategory c : categories) {
//            Parameter p = c.getAxis().getParameter();
//            Integer designsCount = designsCountMap.get(p);
//            if(designsCount == null) {
//                designsCount = 0;
//                designsCountMap.put(p, designsCount);
//            }
//            designsCount += c.getDesigns().size();
//        }
//        Parameter min = null;
//        int minDesignsCount = Integer.MAX_VALUE;
//        for(Parameter p : designsCountMap.keySet()) {
//            int designsCount = designsCountMap.get(p); 
//            if(minDesignsCount > designsCount) {
//                minDesignsCount = designsCount;
//                min = p;
//            }
//        }
//        //For those designs test the other parameters
//        //TODO 
//    }
}
