/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.xdat.chart;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;
import org.xdat.UserPreferences;
import org.xdat.data.DataSheet;
import org.xdat.data.Design;

/**
 *
 * @author cmolikl
 */
public class ParallelSetsFilterAxis extends ParallelSetsAxis {

    double noFilterDesigns = 0;
    private AxisCategory noFilterCategory;

    public ParallelSetsFilterAxis(ParallelSetsChart chart) {
        this.parallelSetsChart = chart;
        selected = false;
        expanded = false;
        this.parallelSetsChart = chart;
        coordinatesAxis = false;
    }

    public AxisCategory getNoFilterCategory() {
        return noFilterCategory;
    }

    public void setNoFilterCategory(AxisCategory noFilterCategory) {
        this.noFilterCategory = noFilterCategory;
    }

    @Override
    public String getName() {
        return "";
    }

    public void setNoFilterDesigns(double d) {
        noFilterDesigns = d;
    }

    @Override
    public int getNumberOfAllCategories() {
        return getNumberOfActiveCategories();
    }

    @Override
    public int getNumberOfActiveCategories() {
        int result = categories.size();
        if(noFilterDesigns > 0) {
            result++;
        }
        return result;
    }

    @Override
    protected void drawCategoryAxis(Graphics2D g) {
        // Categories
        List<AxisCategory> categories = getCategories();
        int i = 0;
        double lastYPos = parallelSetsChart.getAxisTopPos();
        for (AxisCategory category : categories) {
            double yDown = category.getyUpPosition() + category.getHeight() + AxisCategory.getYOffset();
            if(yDown > lastYPos) {
                lastYPos = yDown;
            }

            ParallelSetsFilter filter = ((FilterAxisCategory) category).getFilter();
            // Selected category
            if (filter == parallelSetsChart.getSelectedFilter()) {
                g.setStroke(new BasicStroke(2));
            }
            g.setColor(filter.getColor());
            g.fillRect(getPosition().x, category.getyUpPosition(), getBoxWidth(), category.getHeight());
            g.setColor(Color.black);
            g.drawRect(getPosition().x, category.getyUpPosition(), getBoxWidth(), category.getHeight());
            g.setStroke(new BasicStroke());

            UserPreferences pref = UserPreferences.getInstance();
            int fontSize = pref.getParallelCoordinatesAxisTicLabelFontSize();
            g.setColor(Color.black);

            Font font = new Font("SansSerif", Font.PLAIN, fontSize);
            g.setFont(font);
            //Rectangle2D bounds = font.getStringBounds("x", fontContext);
            float textHeight = 7.5f;
            String cLabel = category.getLabel();
            String[] rows = cLabel.split("\\n");
            float labelX = getPosition().x + getBoxWidth() + 5;
            float labelY = (float) (category.getyUpPosition() + category.getHeight() / 2f + textHeight / 2f - 12f * (rows.length - 1) / 2f);
            for (String row : rows) {
                g.drawString(row, labelX, labelY);
                labelY += 12f;
            }
        }
        if (noFilterDesigns > 0) {
            int y = (int) lastYPos;

            int axisHeight = getChart().getAxisHeight();
            double usableHeight = axisHeight - (getNumberOfActiveCategories() - 1) * AxisCategory.getYOffset();

            int height = (int) (usableHeight * noFilterDesigns);

            if (parallelSetsChart.getSelectedFilter() == null) {
                g.setStroke(new BasicStroke(2));
                g.setColor(Color.BLACK);
                g.drawRect(getPosition().x, y, getBoxWidth(), height);
                g.setStroke(new BasicStroke());
            }

            g.setColor(ParallelSetsChart.NO_FILTER_COLOR);
            g.fillRect(getPosition().x, y, getBoxWidth(), height);
        }
    }

    @Override
    public void moveCategory(AxisCategory category, int newIndex) {
        super.moveCategory(category, newIndex);
        // move filter
        List<ParallelSetsFilter> filters = parallelSetsChart.getFilters();
        ParallelSetsFilter filter = ((FilterAxisCategory) category).getFilter();
        filters.remove(filter);
        filters.add(newIndex, filter);

        parallelSetsChart.setBuildDesignSetsRequired(true);
    }

    @Override
    public void update(int xPosition) {
        // reverse order of categories
        double yPosition = getChart().getAxisTopPos();
        int axisHeight = getChart().getAxisHeight();

        setPosition(new Point(xPosition, (int) yPosition));

        int numberOfCategories = getNumberOfActiveCategories() - 1;
        double usableHeight = axisHeight -  numberOfCategories* AxisCategory.getYOffset();

        for (AxisCategory category : getCategories()) {
            if(!category.isActive() && !isCoordinatesAxis()) continue;
            category.setyUpPosition((int) yPosition);

            double height = usableHeight * category.getHeightRatio();
            category.setHeight((int) height);

            yPosition += height;
            yPosition += AxisCategory.getYOffset();
        }
    }

    @Override
    public void updateCategories(int coordAxisScaleMode) {
        DataSheet dataSheet = parallelSetsChart.getDataSheet(); //getDataSheet();
        double heightSum = 0;
        for(AxisCategory cat : getCategories()) {
            AxisCategoryCondition c = cat.getCondition();
            List<Design> designsInCat = cat.getDesigns();
            if(designsInCat == null) {
                designsInCat = new ArrayList<>();
            }
            double designsCount = (double) (dataSheet.getDesignCount() - dataSheet.getInactiveDesigns());
            double height = 0;
            height = (double) designsInCat.size() / designsCount;
            heightSum += height;
            cat.setHeightRatio(height);
        }
        if(heightSum != 1.0) {
            System.out.println("Height sumfor axis " + getName() + "is " + heightSum);
        }
    }

}
