/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.xdat.example;

import java.io.IOException;
import java.util.HashSet;
import javax.swing.ProgressMonitor;
import org.xdat.Main;
import org.xdat.UserPreferences;
import org.xdat.chart.AxisCategoryOrCondition;
import org.xdat.chart.AxisCategoryStringCondition;
import org.xdat.chart.ParallelSetsAxis;
import org.xdat.chart.ParallelSetsChart;
import org.xdat.data.DataSheet;
import org.xdat.data.Design;
import org.xdat.data.Parameter;
import org.xdat.exceptions.NoParametersDefinedException;
import org.xdat.gui.frames.ChartFrame;

/**
 *
 * @author cmolikl
 */
public class CensusIncome {
    public static void main(String... args) {
        try {
            String filePath = args[0];
            String delimiter = args[1];
            Boolean header = Boolean.parseBoolean(args[2]);
            
            Main main = new Main();
            
            UserPreferences userPreferences = UserPreferences.getInstance();
            userPreferences.setDelimiter(delimiter);
            userPreferences.setTreatConsecutiveAsOne(false);
           
            ProgressMonitor progressMonitor = new ProgressMonitor(main, "", "Importing Data...", 0, 100);
            progressMonitor.setProgress(0);

            DataSheet dataSheet = new DataSheet(filePath, header, main, progressMonitor);
            
            HashSet<String> params = new HashSet<>();
            params.add("age");
            params.add("income");
            params.add("wage_per_hour");
            params.add("class_of_worker");
            params.add("education");
            params.add("marital_stat");
            params.add("sex");
            params.add("member_of_a_labor_union");
            params.add("family_members_under_18");
            params.add("weeks_worked_in_year");
            
            for(Parameter p : dataSheet.getParameters()) {
                if(!params.contains(p.getName())) {
                    p.setActive(false);
                }
            }
            
            aggregate(dataSheet);
            
            main.setDataSheet(dataSheet);
            main.repaint();
            
            progressMonitor = new ProgressMonitor(main, "", "Creating chart...", 0, 100);
            progressMonitor.setProgress(0);
            
            ParallelSetsChart chart = new ParallelSetsChart(dataSheet, null, progressMonitor, main.getUniqueChartId(ParallelSetsChart.class));
            //((ParallelSetsAxis) chart.getAxis("age")).getCategoryFromActive(4).setActive(false);
            //((ParallelSetsAxis) chart.getAxis("cylinders")).getCategory(2).setActive(false);
            //((ParallelSetsAxis) chart.getAxis("cylinders")).getCategory(3).setActive(false);
            //((ParallelSetsAxis) chart.getAxis("cylinders")).getCategory(4).setActive(false);
            //chart.update();
            //chart.getAxis("name").setActive(false);
	    ChartFrame chartFrame = new ChartFrame(main, chart);
           
            main.getCurrentSession().addChart(chart); 
        }
        catch(IOException e) {
            e.printStackTrace();
        } catch (NoParametersDefinedException ex) {
            ex.printStackTrace();
        }
    }
    
    public static void aggregate(DataSheet ds) {
        Parameter edu = ds.getParameter("education");
//        Parameter cylinders = ds.getParameter("cylinders");
//        Parameter weight = ds.getParameter("weight (lb)");
//        Parameter year = ds.getParameter("year");
        
//        AxisCategoryRangeCondition eC = new AxisCategoryRangeCondition(16.52, 24.04, economy);
//        AxisCategoryStringCondition cC = new AxisCategoryStringCondition("6.0", cylinders);
//        AxisCategoryRangeCondition wC1 = new AxisCategoryRangeCondition(3023.8, 3729.2, weight);
//        AxisCategoryRangeCondition wC2 = new AxisCategoryRangeCondition(2318.4, 3023.8, weight);
        
        AxisCategoryStringCondition eCcollNoD = new AxisCategoryStringCondition("Some college but no degree", edu);
        AxisCategoryStringCondition eCprof = new AxisCategoryStringCondition("Prof school degree (MD DDS DVM LLB JD)", edu);
        AxisCategoryStringCondition eCmaster = new AxisCategoryStringCondition("Masters degree(MA MS MEng MEd MSW MBA)", edu);
        AxisCategoryStringCondition eC0g = new AxisCategoryStringCondition("Less than 1st grade", edu);
        AxisCategoryStringCondition eChigh = new AxisCategoryStringCondition("High school graduate", edu);
        AxisCategoryStringCondition eCphd = new AxisCategoryStringCondition("Doctorate degree(PhD EdD)", edu);
        AxisCategoryStringCondition eCchild = new AxisCategoryStringCondition("Children", edu);
        AxisCategoryStringCondition eCbc = new AxisCategoryStringCondition("Bachelors degree(BA AB BS)", edu);
        AxisCategoryStringCondition eCvoc = new AxisCategoryStringCondition("Associates degree-occup /vocational", edu);
        AxisCategoryStringCondition eCaca = new AxisCategoryStringCondition("Associates degree-academic program", edu);
        AxisCategoryStringCondition eC9g = new AxisCategoryStringCondition("9th grade", edu);
        AxisCategoryStringCondition eC78g = new AxisCategoryStringCondition("7th and 8th grade", edu);
        AxisCategoryStringCondition eC56g = new AxisCategoryStringCondition("5th or 6th grade", edu);
        AxisCategoryStringCondition eC1234g = new AxisCategoryStringCondition("1st 2nd 3rd or 4th grade", edu);
        AxisCategoryStringCondition eC12gNoD = new AxisCategoryStringCondition("12th grade no diploma", edu);
        AxisCategoryStringCondition eC11g = new AxisCategoryStringCondition("11th grade", edu);
        AxisCategoryStringCondition eC10g = new AxisCategoryStringCondition("10th grade", edu);
        
        AxisCategoryOrCondition eCinPreschool = new AxisCategoryOrCondition(eC0g, eCchild);
        AxisCategoryOrCondition eCinElementary = new AxisCategoryOrCondition(eC1234g, eC56g);
        AxisCategoryOrCondition eCinHigh = new AxisCategoryOrCondition(eC78g, eC9g, eC10g, eC11g, eC12gNoD);
        AxisCategoryOrCondition eChighGrad = new AxisCategoryOrCondition(eChigh, eCcollNoD);
        AxisCategoryOrCondition eCvocational = new AxisCategoryOrCondition(eCvoc, eCaca);
        
        
        Parameter e = new Parameter("education_", ds);
        e.numeric = false;
        ds.addParameter(e);
        
        edu.setParent(e);
        
        for(int i = 0; i < ds.getDesignCount(); i++) {
            Design d = ds.getDesign(i);
            
            if(eCinPreschool.test(d)) {
                d.setValue(e, "Preschool education");
            }
            else if(eCinElementary.test(d)) {
                d.setValue(e, "Elementary not finished");
            }
            else if(eCinHigh.test(d)) {
                d.setValue(e, "Highschool not finished");
            }
            else if(eChighGrad.test(d)) {
                d.setValue(e, "Higschool graduate");
            }
            else if(eCvocational.test(d)) {
                d.setValue(e, "Associate degree");
            }
            else if(eCbc.test(d)) {
                d.setValue(e, "Bachelors degree");
            }
            else if(eCmaster.test(d)) {
                d.setValue(e, "Masters degree");
            }
            else if(eCprof.test(d)) {
                d.setValue(e, "Professional school degree");
            }
            else if(eCphd.test(d)) {
                d.setValue(e, "Doctoral degree");
            }
            else {
                d.setValue(e, "Other");
            }
        }
    }
}
