/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.xdat.example;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.ProgressMonitor;
import org.xdat.Main;
import org.xdat.UserPreferences;
import org.xdat.chart.AxisCategoryOrCondition;
import org.xdat.chart.AxisCategoryStringCondition;
import org.xdat.chart.ParallelSetsChart;
import org.xdat.data.DataSheet;
import org.xdat.data.Design;
import org.xdat.data.Parameter;
import org.xdat.exceptions.NoParametersDefinedException;
import org.xdat.gui.frames.ChartFrame;
import org.xdat.gui.tables.DataSheetTableColumnModel;

/**
 *
 * @author cmolikl
 */
public class InfoVis {
    public static void main(String... args) {
        try {
            String filePath = args[0];
            String delimiter = args[1];
            Boolean header = Boolean.parseBoolean(args[2]);
            
            Main main = new Main();
            
            UserPreferences userPreferences = UserPreferences.getInstance();
            userPreferences.setDelimiter(delimiter);
            userPreferences.setTreatConsecutiveAsOne(false);
           
            ProgressMonitor progressMonitor = new ProgressMonitor(main, "", "Importing Data...", 0, 100);
            progressMonitor.setProgress(0);

            DataSheet dataSheet = new DataSheet(filePath, header, main, progressMonitor);
   
            //dataSheet.removeParameter("Title");
            dataSheet.removeParameter("Category");
            //dataSheet.removeParameter("Author");
            dataSheet.removeParameter("Keyword");
            dataSheet.removeParameter("Column 24");
            dataSheet.removeParameter("Column 25");
            
            Parameter p = dataSheet.getParameter("Year");
            p.setNumeric(false);
            
            //aggregate(dataSheet);
            
            main.setDataSheet(dataSheet);
            main.repaint();
            
            progressMonitor = new ProgressMonitor(main, "", "Creating chart...", 0, 100);
            progressMonitor.setProgress(0);

            // TODO get ColumnModel for chart
            ParallelSetsChart chart = new ParallelSetsChart(dataSheet, null, progressMonitor, main.getUniqueChartId(ParallelSetsChart.class));
            chart.getAxis("Title").setActive(false);
            //chart.getAxis("Session").setActive(false);
            chart.getAxis("Author").setActive(false);
            chart.update();
	    ChartFrame chartFrame = new ChartFrame(main, chart);
            main.getCurrentSession().addChart(chart);
            
        }
        catch(IOException e) {
            e.printStackTrace();
        } catch (NoParametersDefinedException ex) {
            ex.printStackTrace();
        }
    }
    
    public static void aggregate(DataSheet ds) {
        ArrayList<Parameter> metrics = new ArrayList<>();
        metrics.add(ds.getParameter("Speed"));
        metrics.add(ds.getParameter("Accuracy"));
        metrics.add(ds.getParameter("Scability"));
        metrics.add(ds.getParameter("Efficiency"));
        metrics.add(ds.getParameter("Effectiveness"));
        metrics.add(ds.getParameter("Satisfaction"));
        metrics.add(ds.getParameter("Utility"));
        metrics.add(ds.getParameter("Learnability"));
        metrics.add(ds.getParameter("Adoption"));
        metrics.add(ds.getParameter("Productivity"));
        
        ArrayList<Parameter> methods = new ArrayList<>();
        methods.add(ds.getParameter("Computational experiments"));
        methods.add(ds.getParameter("Subjective experiments"));
        methods.add(ds.getParameter("Case study"));
        methods.add(ds.getParameter("Comparison with other method"));
        methods.add(ds.getParameter("Empirical analysis"));
        methods.add(ds.getParameter("User Study"));
        methods.add(ds.getParameter("Theoretical Analysis"));
        
        Parameter m = new Parameter("Metrics", ds);
        m.numeric = false;
        ds.addParameter(m);
        
        Parameter meth = new Parameter("Methods", ds);
        meth.numeric = false;
        ds.addParameter(meth);
        
        Parameter ses = new Parameter("Session_", ds);
        ses.numeric = false;
        ds.addParameter(ses);
        
        for(Parameter p : metrics) {
            p.setParent(m);
        }
        
        for(Parameter p : methods) {
            p.setParent(meth);
        }
        
        Parameter session = ds.getParameter("Session");
        HashMap<AxisCategoryOrCondition, String> sessionMap = new HashMap<>();
        
        AxisCategoryOrCondition c = new AxisCategoryOrCondition();
        AxisCategoryStringCondition sc = new AxisCategoryStringCondition("Tree and Treemap Applications", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Applications", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("\"Applications: Geo, Bio, and Time\"", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Social Applications", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Education and Popular Applications", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Application Areas", session);
        c.addCondition(sc);
        sessionMap.put(c, "Applications");
        
        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Data Perception", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Perception and Cognition", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Representation and Perception", session);
        c.addCondition(sc);
        sessionMap.put(c, "Perception");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Geographic Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Maps and Geovisualization", session);
        c.addCondition(sc);
        sessionMap.put(c, "Geo Visualization");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Network Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Graph Drawing", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Tree and Graph Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Graph Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Graphs", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Graphs and Networks", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Graph Exploration", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("\"Time, Trees and Graphs\"", session);
        c.addCondition(sc);
        sessionMap.put(c, "Graph Visualization");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("InfoVis for the Masses", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Visualization for the Masses", session);
        c.addCondition(sc);
        sessionMap.put(c, "Visualization for the Masses");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Novel Techniques", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Techniques", session);
        c.addCondition(sc);
        sessionMap.put(c, "Techniques");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Interaction and Animation", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Â Interaction", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Interaction", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Overviews and Detail", session);
        c.addCondition(sc);
        sessionMap.put(c, "Interaction");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Evaluation", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Evaluation And Methodology", session);
        c.addCondition(sc);
        sessionMap.put(c, "Evaluation");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Text Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Text and Time", session);
        c.addCondition(sc);
        sessionMap.put(c, "Text Visualization");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Design", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Design Spaces", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Sketching and Designing Visualizations", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Defining the Design Space", session);
        c.addCondition(sc);
        sessionMap.put(c, "Design Space");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Models and Theories", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Theory and Foundations", session);
        c.addCondition(sc);
        sessionMap.put(c, "Theory of Vis");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Multidimensional Data Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Multi-dimensionsal Visualizaton", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Multidimensional Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("High-Dimensional Data", session);
        c.addCondition(sc);
        sessionMap.put(c, "Multidimensional Visualization");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Systems", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Systems and Frameworks", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Frameworks", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Systems and Sets", session);
        c.addCondition(sc);
        sessionMap.put(c, "Systems");

        c = new AxisCategoryOrCondition();
        sc = new AxisCategoryStringCondition("Ranking and Ordering Data", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Bioinformatics Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Space and Time", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Collaborative Visualization", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Times and Trees", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Ordinal and Categorical Data", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Space and Maps", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Storytelling and Presentation", session);
        c.addCondition(sc);
        sc = new AxisCategoryStringCondition("Best Paper Award", session);
        c.addCondition(sc);
        sessionMap.put(c, "Other");
        
        for(int i = 0; i < ds.getDesignCount(); i++) {
            Design d = ds.getDesign(i);
            int count = 0;
            for(Parameter p : metrics) {
                if(d.getStringValue(p).equals("1.0")) {
                    count++;
                }
            }
            d.setValue(m, "" + count);
            
            count = 0;
            for(Parameter p : methods) {
                if(d.getStringValue(p).equals("1.0")) {
                    count++;
                }
            }
            d.setValue(meth, "" + count);
            
            boolean isInCat = false;
            for(AxisCategoryOrCondition oC : sessionMap.keySet()) {
                if(oC.test(d)) {
                    String cat = sessionMap.get(oC);
                    d.setValue(ses, cat);
                    isInCat = true;
                }
            }
            if(!isInCat) {
                d.setValue(ses, "Other");
            }
        }
    }
}
