package org.xdat.gui.dialogs;

import org.xdat.Main;
import org.xdat.chart.*;
import org.xdat.gui.frames.ChartFrame;
import org.xdat.gui.menus.parallelSetsChart.CustomAxisMenuBar;
import org.xdat.gui.panels.ParallelSetsChartPanel;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

public class CreateCustomAxisDialog extends JDialog {

    public static final int EMPTY_SELECTION = 0;
    public static final int CATEGORY_SELECTION = 1;
    public static final int PARENT_CONDITION_SELECTION = 2;
    public static final int LEAF_CONDITION_SELECTION = 3;

    private JTree tree;
    private DefaultMutableTreeNode top;
    private ParallelSetsChart chart;
    private ParallelSetsChartPanel chartPanel;

    private List<AxisCategory> categories;

    public CreateCustomAxisDialog(ChartFrame parent, Main mainWindow){
        super(parent, "Create Custom Axis");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        this.chart = (ParallelSetsChart) parent.getChart();
        this.chartPanel = (ParallelSetsChartPanel) parent.getChartPanel();
        categories = new ArrayList<>();

        setLayout(new BorderLayout());
        JPanel mainPanel = new JPanel(new BorderLayout());

        JPanel menuPanel = new JPanel(new BorderLayout());
        CustomAxisMenuBar menuBar = new CustomAxisMenuBar(this, chart);
        menuPanel.add(menuBar, BorderLayout.CENTER);
        mainPanel.add(menuPanel, BorderLayout.NORTH);

        top = new DefaultMutableTreeNode("Custom Axis");
        tree = new JTree(top);
//        tree.setPreferredSize(new Dimension(500, 200));
//        treeModel = new DefaultTreeModel(top);
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            @Override
            public void valueChanged(TreeSelectionEvent e) {
                menuBar.selectedNodeChanged();
            }
        });
        JScrollPane treeView = new JScrollPane(tree);
        mainPanel.add(treeView, BorderLayout.CENTER);

        JPanel buttonsPanel = new JPanel(new FlowLayout());
        JButton bCreateAxis = new JButton("Create axis");
        bCreateAxis.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                createAxis();
            }
        });
        buttonsPanel.add(bCreateAxis);
        mainPanel.add(buttonsPanel, BorderLayout.SOUTH);

        add(mainPanel, BorderLayout.CENTER);

        Dimension parentSize = parent.getSize();
        setSize(new Dimension(500, 400));
        int xPos = Math.max(parent.getX()+(int) (0.5 * (parentSize.width - getSize().width)), 0);
        int yPos = Math.max(parent.getY()+((int) (0.5 * (parentSize.height - getSize().height))), 0);
        setLocation(xPos, yPos);
        this.setVisible(true);
    }

    public void addNewCategory() {
        DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
        String categoryLabel = JOptionPane.showInputDialog(this, "Category name:");
        AxisCategory c1 = new AxisCategory(new AxisCategoryAllCondition(categoryLabel), 1, new ArrayList<>(), null);
        DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(c1);
        model.insertNodeInto(categoryNode, top, top.getChildCount());
        tree.expandPath(new TreePath(top.getPath()));

        categories.add(c1);
    }

    public void setCategoryCondition(AxisCategory conditionCategory) {
        DefaultMutableTreeNode sourceNode = (DefaultMutableTreeNode) tree.getSelectionPath().getLastPathComponent();
        AxisCategory sourceCategory = (AxisCategory) sourceNode.getUserObject();
        sourceCategory.setCondition(conditionCategory.getCondition());

        DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
        if(sourceNode.getChildCount() > 0){
            model.removeNodeFromParent(sourceNode.getFirstLeaf());
        }
        DefaultMutableTreeNode conditionNode = new DefaultMutableTreeNode(conditionCategory.getCondition());
        model.insertNodeInto(conditionNode, sourceNode, 0);
        tree.expandPath(new TreePath(sourceNode.getPath()));
    }

    public void setLogicalCondition(AxisCategoryCondition condition){
        DefaultMutableTreeNode sourceNode = (DefaultMutableTreeNode) tree.getSelectionPath().getLastPathComponent();
        AxisCategory sourceCategory = (AxisCategory) sourceNode.getUserObject();
        sourceCategory.setCondition(condition);

        DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
        if(sourceNode.getChildCount() > 0){
            model.removeNodeFromParent(sourceNode.getFirstLeaf());
        }
        DefaultMutableTreeNode conditionNode = new DefaultMutableTreeNode(condition);
        model.insertNodeInto(conditionNode, sourceNode, 0);
        tree.expandPath(new TreePath(sourceNode.getPath()));
    }

    public void addLogicalSubcondition(AxisCategoryCondition condition) {
        DefaultMutableTreeNode sourceNode = (DefaultMutableTreeNode) tree.getSelectionPath().getLastPathComponent();
        AxisCategoryCondition parentCondition = (AxisCategoryCondition) sourceNode.getUserObject();
        DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
        if(parentCondition.getClass() == AxisCategoryAndCondition.class) {
            addSubconditionToAnd((AxisCategoryAndCondition) parentCondition, condition);
        } else if(parentCondition.getClass() == AxisCategoryOrCondition.class) {
            addSubconditionToOr((AxisCategoryOrCondition) parentCondition, condition);
        } else if(parentCondition.getClass() == AxisCategoryNotCondition.class) {
            addSubconditionToNot((AxisCategoryNotCondition) parentCondition, condition);
            if(sourceNode.getChildCount() > 0){
                model.removeNodeFromParent(sourceNode.getFirstLeaf());
            }
        }

        DefaultMutableTreeNode conditionNode = new DefaultMutableTreeNode(condition);
        model.insertNodeInto(conditionNode, sourceNode, sourceNode.getChildCount());
        tree.expandPath(new TreePath(sourceNode.getPath()));
    }

    public void addCategorySubcondition(AxisCategory conditionCategory) {
        DefaultMutableTreeNode sourceNode = (DefaultMutableTreeNode) tree.getSelectionPath().getLastPathComponent();
        AxisCategoryCondition parentCondition = (AxisCategoryCondition) sourceNode.getUserObject();
        DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
        if(parentCondition.getClass() == AxisCategoryAndCondition.class) {
            addSubconditionToAnd((AxisCategoryAndCondition) parentCondition, conditionCategory.getCondition());
        } else if(parentCondition.getClass() == AxisCategoryOrCondition.class) {
            addSubconditionToOr((AxisCategoryOrCondition) parentCondition, conditionCategory.getCondition());
        } else if(parentCondition.getClass() == AxisCategoryNotCondition.class) {
            addSubconditionToNot((AxisCategoryNotCondition) parentCondition, conditionCategory.getCondition());
            if(sourceNode.getChildCount() > 0){
                model.removeNodeFromParent(sourceNode.getFirstLeaf());
            }
        }

        DefaultMutableTreeNode conditionNode = new DefaultMutableTreeNode(conditionCategory.getCondition());
        model.insertNodeInto(conditionNode, sourceNode, sourceNode.getChildCount());
        tree.expandPath(new TreePath(sourceNode.getPath()));
    }

    public void addSubconditionToAnd(AxisCategoryAndCondition parentCondition, AxisCategoryCondition subcondition) {
        parentCondition.addCondition(subcondition);
    }

    public void addSubconditionToOr(AxisCategoryOrCondition parentCondition, AxisCategoryCondition subcondition) {
        parentCondition.addCondition(subcondition);
    }

    public void addSubconditionToNot(AxisCategoryNotCondition parentCondition, AxisCategoryCondition subcondition) {
        parentCondition.setCondition(subcondition);
    }

    private void createAxis() {
        chart.createCustomAxisByDialog(categories);
        chartPanel.updateLayout();
        chartPanel.repaint();
        this.dispose();
    }

    public int getSelectionMode() {
        TreePath selectionPath = tree.getSelectionPath();
        if(selectionPath == null) {
            return EMPTY_SELECTION;
        }
        DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
        Object selectedObject = selectedNode.getUserObject();
        if(selectedObject instanceof AxisCategory) {
            return CATEGORY_SELECTION;
        }
        if(selectedObject instanceof AxisCategoryCondition) {
            if(selectedObject.getClass() == AxisCategoryAndCondition.class
                    || selectedObject.getClass() == AxisCategoryOrCondition.class
                    || selectedObject.getClass() == AxisCategoryNotCondition.class) {
                return PARENT_CONDITION_SELECTION;
            } else {
                return LEAF_CONDITION_SELECTION;
            }
        }
        return EMPTY_SELECTION;
    }
}
