package org.xdat.gui.menus.parallelSetsChart;

import org.xdat.chart.*;
import org.xdat.gui.dialogs.CreateCustomAxisDialog;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class CustomAxisMenuBar extends JMenuBar {

    private JMenu setCondition;
    JMenuItem renameCategory;
    private JMenu addSubcondition;

    CreateCustomAxisDialog dialog;

    public CustomAxisMenuBar(CreateCustomAxisDialog dialog, ParallelSetsChart chart){
        this.dialog = dialog;
        //      AXIS MENU
        JMenu axisMenu = new JMenu("Axis");
        // Rename axis
        JMenuItem renameAxis = new JMenuItem("Rename axis");
        axisMenu.add(renameAxis);
        // Clear all
        JMenuItem clear = new JMenuItem("Clear all");
        axisMenu.add(clear);
        add(axisMenu);

        //      CATEGORY MENU
        JMenu categoryMenu = new JMenu("Category");
        // Add category
        JMenuItem addNewCategory = new JMenuItem("Add new category");
        addNewCategory.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dialog.addNewCategory();
            }
        });
        categoryMenu.add(addNewCategory);
        // Rename category
        renameCategory = new JMenuItem("Rename category");
        categoryMenu.add(renameCategory);
        // Remove category
        //JMenuItem removeCategory = new JMenuItem("Remove category");
        //categoryMenu.add(removeCategory);
        add(categoryMenu);

        //      CONDITION MENU
        JMenu conditionMenu = new JMenu("Condition");
        // Set condition
        setCondition = new JMenu("Set condition");
        setCondition.setEnabled(false);
        JMenu logicalCond = new JMenu("Logical condition");
        JMenuItem andCond = new JMenuItem("And");
        andCond.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dialog.setLogicalCondition(new AxisCategoryAndCondition());
            }
        });
        logicalCond.add(andCond);
        JMenuItem orCond = new JMenuItem("Or");
        orCond.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dialog.setLogicalCondition(new AxisCategoryOrCondition());
            }
        });
        logicalCond.add(orCond);
        JMenuItem notCond = new JMenuItem("Not");
        notCond.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dialog.setLogicalCondition(new AxisCategoryNotCondition());
            }
        });
        logicalCond.add(notCond);
        setCondition.add(logicalCond);
        JMenu catCond = new JMenu("Axis category condition");
        for (ParallelSetsAxis axis : chart.getAllAxes()){
            JMenu axisCondMenu = new JMenu(axis.getName());
            for(AxisCategory category : axis.getCategories()){
                JMenuItem catCondItem = new JMenuItem(category.getLabel());
                catCondItem.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        dialog.setCategoryCondition(category);
                    }
                });
                axisCondMenu.add(catCondItem);
            }
            catCond.add(axisCondMenu);
        }
        setCondition.add(catCond);
        conditionMenu.add(setCondition);
        // Add subcondition
        addSubcondition = new JMenu("Add sub-condition");
        addSubcondition.setEnabled(false);
        JMenu logicalSubCond = new JMenu("Logical condition");
        JMenuItem andSubCond = new JMenuItem("And");
        andSubCond.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dialog.addLogicalSubcondition(new AxisCategoryAndCondition());
            }
        });
        logicalSubCond.add(andSubCond);
        JMenuItem orSubCond = new JMenuItem("Or");
        orSubCond.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dialog.addLogicalSubcondition(new AxisCategoryOrCondition());
            }
        });
        logicalSubCond.add(orSubCond);
        JMenuItem notSubCond = new JMenuItem("Not");
        notSubCond.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dialog.addLogicalSubcondition(new AxisCategoryNotCondition());
            }
        });
        logicalSubCond.add(notSubCond);
        addSubcondition.add(logicalSubCond);
        JMenu catSubCond = new JMenu("Axis category condition");
        for (ParallelSetsAxis axis : chart.getAllAxes()){
            JMenu axisCondMenu = new JMenu(axis.getName());
            for(AxisCategory category : axis.getCategories()){
                JMenuItem catCondItem = new JMenuItem(category.getLabel());
                catCondItem.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        dialog.addCategorySubcondition(category);
                    }
                });
                axisCondMenu.add(catCondItem);
            }
            catSubCond.add(axisCondMenu);
        }
        addSubcondition.add(catSubCond);
        conditionMenu.add(addSubcondition);
        add(conditionMenu);
    }

    public void selectedNodeChanged() {
        if(dialog.getSelectionMode() == CreateCustomAxisDialog.CATEGORY_SELECTION) {
            setCondition.setEnabled(true);
            renameCategory.setEnabled(true);
        } else {
            setCondition.setEnabled(false);
            renameCategory.setEnabled(false);
        }

        if(dialog.getSelectionMode() == CreateCustomAxisDialog.PARENT_CONDITION_SELECTION) {
            addSubcondition.setEnabled(true);
        } else {
            addSubcondition.setEnabled(false);
        }
    }

}
