
package org.xdat.gui.menus.parallelSetsChart;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import org.xdat.Main;
import org.xdat.chart.ParallelSetsChart;
import org.xdat.gui.dialogs.ParallelSetsDisplaySettingsDialog;
import org.xdat.gui.frames.ChartFrame;
import org.xdat.gui.menus.ChartFrameMenuBar;
import org.xdat.gui.menus.parallelCoordinatesChart.ParallelCoordinatesChartFrameSelectParametersMenu;
import org.xdat.gui.panels.ParallelSetsChartPanel;


public class ParallelSetsChartFrameMenuBar extends ChartFrameMenuBar {

        private ParallelSetsChartPanel chartPanel;
        private ParallelSetsChart chart;

        private JRadioButtonMenuItem categoryLayout;
        private JRadioButtonMenuItem colorLayout;
        private JRadioButtonMenuItem treeLayout;
        private JCheckBoxMenuItem axisTreeCheckbox;
        private JRadioButtonMenuItem noScaling;
        private JRadioButtonMenuItem scalingCategories;
        private JRadioButtonMenuItem scalingAxis;
        private JCheckBoxMenuItem aggregateValues;
        private JCheckBoxMenuItem showSortButtonCheckbox;
        
	public ParallelSetsChartFrameMenuBar(Main mainWindow, ChartFrame chartFrame, ParallelSetsChart chart) {
		super(chartFrame, chart);
                chartPanel = (ParallelSetsChartPanel) chartFrame.getChartPanel();
                this.chart = chart;
                
		this.add(new ParallelCoordinatesChartFrameSelectParametersMenu(mainWindow, chartFrame, chart));

                JMenu layoutMenu = new JMenu("Layout");
                categoryLayout = new JRadioButtonMenuItem("Bundled Layout");
                categoryLayout.setSelected(false);
                treeLayout = new JRadioButtonMenuItem("Tree Layout");
                treeLayout.setSelected(false);
                colorLayout = new JRadioButtonMenuItem("Color Group Layout");
                colorLayout.setSelected(false);
                
                categoryLayout.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        categoryLayoutActionPerformed(e);
                    }
                });
                treeLayout.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        treeLayoutActionPerformed(e);
                    }
                });
                colorLayout.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        colorLayoutActionPerformed(e);
                    }
                });
                
                layoutMenu.add(categoryLayout);
                //layoutMenu.add(colorLayout);
                layoutMenu.add(treeLayout);
                this.add(layoutMenu);
                
                // init
                switch(chart.getLayoutMode()){
                    case ParallelSetsChart.BUNDLED_LAYOUT:
                        categoryLayout.setSelected(true);
                        break;
                    case ParallelSetsChart.TREE_LAYOUT:
                        treeLayout.setSelected(true);
                        break;
                    case ParallelSetsChart.BUNDLED_COLOR_LAYOUT:
                        colorLayout.setSelected(true);
                        break;
                }
                
                
                
                JMenu axisMenu = new JMenu("Axis");
                axisTreeCheckbox = new JCheckBoxMenuItem("Tree inside axis");
                
                axisMenu.addMenuListener(new MenuListener() {
                    @Override
                    public void menuSelected(MenuEvent e) {
                        updateAxisTreeCheckbox();
                    }

                    @Override
                    public void menuDeselected(MenuEvent e) {
                    }

                    @Override
                    public void menuCanceled(MenuEvent e) {
                    }
                });
                axisTreeCheckbox.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        axisTreeCheckboxActionPerformed(e);
                    }
                });
                
                //axisTreeCheckbox.add
                
                axisMenu.add(axisTreeCheckbox);
                
                showSortButtonCheckbox = new JCheckBoxMenuItem("Show sort button");
                showSortButtonCheckbox.setSelected(true);
                showSortButtonCheckbox.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        showSortButtonCheckboxActionPerformed(e);
                    }
                });
                axisMenu.add(showSortButtonCheckbox);
                
                JMenuItem zoomInMenuItem = new JMenuItem("Zoom in");
                zoomInMenuItem.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        zoomInMenuItem();
                    }
                });
                axisMenu.add(zoomInMenuItem);
                
                JMenuItem zoomOutMenuItem = new JMenuItem("Zoom out");
                zoomOutMenuItem.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        zoomOutMenuItem();
                    }
                });
                axisMenu.add(zoomOutMenuItem);
                
                this.add(axisMenu);
                
                JMenu coordinatesAxisMenu = new JMenu("Coordinates axis");
                noScaling = new JRadioButtonMenuItem("No scaling");
                noScaling.setSelected(true);
                scalingCategories = new JRadioButtonMenuItem("Scaling categories");
                scalingCategories.setSelected(false);
                scalingAxis = new JRadioButtonMenuItem("Scaling axis");
                scalingAxis.setSelected(false);
                
                noScaling.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        noScalingActionPerformed(e);
                    }
                });
                scalingCategories.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        scalingCategoriesActionPerformed(e);
                    }
                });
                scalingAxis.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        scalingAxisActionPerformed(e);
                    }
                });
                
                coordinatesAxisMenu.add(noScaling);
                coordinatesAxisMenu.add(scalingCategories);
                coordinatesAxisMenu.add(scalingAxis);
                
                aggregateValues = new JCheckBoxMenuItem("Average values");
                aggregateValues.setSelected(chartPanel.getDrawAggregatedValues());
                aggregateValues.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        chartPanel.setDrawAggregatedValues(!chartPanel.getDrawAggregatedValues());
                        chartPanel.updateLayout();
                        chartPanel.repaint();
                    }
                });
                axisMenu.add(aggregateValues);
                
                
                this.add(coordinatesAxisMenu);

                JMenu optionsMenu = new JMenu("Options");
                JMenuItem displaySettings = new JMenuItem("Display Settings");
                displaySettings.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        new ParallelSetsDisplaySettingsDialog(chartFrame, mainWindow);
                    }
                });
                optionsMenu.add(displaySettings);
                this.add(optionsMenu);
                
	}
        
//        private void updateCategoriesScaling(){
//            for (ParallelSetsAxis axis : chart.getAllAxes()) {
//                if(axis.isCoordinatesAxis()){
//                    axis.updateCategoriesHeightRatio();
//                }
//            }
//        }
        
        private void noScalingActionPerformed(ActionEvent e){
            chart.setCoordAxisScaleMode(ParallelSetsChart.COORD_AXIS_NO_SCALING);
            //updateCategoriesScaling();
            
            noScaling.setSelected(true);
            scalingCategories.setSelected(false);
            scalingAxis.setSelected(false);
            
            chartPanel.updateLayout();
            chartPanel.repaint();
        }
        
        private void scalingCategoriesActionPerformed(ActionEvent e){
            chart.setCoordAxisScaleMode(ParallelSetsChart.COORD_AXIS_SCALING_CATEGORIES);
            //updateCategoriesScaling();
            
            noScaling.setSelected(false);
            scalingCategories.setSelected(true);
            scalingAxis.setSelected(false);
            
            chartPanel.updateLayout();
            chartPanel.repaint();
        }
        
        private void scalingAxisActionPerformed(ActionEvent e){
            chart.setCoordAxisScaleMode(ParallelSetsChart.COORD_AXIS_SCALING_AXIS);
            //updateCategoriesScaling();
            
            noScaling.setSelected(false);
            scalingCategories.setSelected(false);
            scalingAxis.setSelected(true);
            
            chartPanel.updateLayout();
            chartPanel.repaint();
        }
        
        private void categoryLayoutActionPerformed(ActionEvent e){
            chartPanel.setLayoutMode(ParallelSetsChart.BUNDLED_LAYOUT);
            categoryLayout.setSelected(true);
            treeLayout.setSelected(false);
            colorLayout.setSelected(false);

            getChartFrame().getSidePanel().updateView(true);
        }
        
        private void treeLayoutActionPerformed(ActionEvent e){
            chartPanel.setLayoutMode(ParallelSetsChart.TREE_LAYOUT);
            treeLayout.setSelected(true);
            categoryLayout.setSelected(false);
            colorLayout.setSelected(false);

            getChartFrame().getSidePanel().updateView(true);
        }
        
        private void colorLayoutActionPerformed(ActionEvent e){
            chartPanel.setLayoutMode(ParallelSetsChart.BUNDLED_COLOR_LAYOUT);
            colorLayout.setSelected(true);
            categoryLayout.setSelected(false);
            treeLayout.setSelected(false);

            getChartFrame().getSidePanel().updateView(true);
        }
        
        private void axisTreeCheckboxActionPerformed(ActionEvent e){
            chart.setInsideParallelogramsOn(axisTreeCheckbox.getState());
            chartPanel.updateLayout();
            chartPanel.repaint();
        }
        
        private void updateAxisTreeCheckbox(){
            if(chartPanel.getLayoutMode() == ParallelSetsChart.TREE_LAYOUT){
                axisTreeCheckbox.setEnabled(false);
            }else{
                axisTreeCheckbox.setEnabled(true);
            }
        }
        
        private void showSortButtonCheckboxActionPerformed(ActionEvent e){
            chart.setSortButtonOn(showSortButtonCheckbox.isSelected());
            chartPanel.repaint();
        }
        
        private void zoomInMenuItem() {
            chart.zoomIn();
            chartPanel.repaint();
        }
        
        private void zoomOutMenuItem() {
            chart.zoomOut();
            chartPanel.repaint();
        }

}
