
package org.xdat.gui.menus.parallelSetsChart;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JCheckBoxMenuItem;

import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import org.xdat.Main;
import org.xdat.actionListeners.parallelCoordinatesChartFrame.ParallelChartContextMenuActionListener;
import org.xdat.chart.ParallelSetsAxis;
import org.xdat.chart.ParallelSetsChart;
import org.xdat.gui.frames.ChartFrame;
import org.xdat.gui.panels.ParallelSetsChartPanel;

public class ParallelSetsContextMenu extends JPopupMenu {
	
        private JCheckBoxMenuItem showCoordinatesAxisItem;
        private JCheckBoxMenuItem axisExpanded;
        private ParallelSetsAxis axis;
        private ParallelSetsChartPanel chartPanel;
        private ParallelSetsChart chart;
    
	public ParallelSetsContextMenu(Main mainWindow, ChartFrame chartFrame, ParallelSetsAxis axis) {
		super();
                
                this.axis = axis;
                chartPanel = (ParallelSetsChartPanel)chartFrame.getChartPanel();
                chart = (ParallelSetsChart) chartPanel.getChart();
                
		this.add(new JLabel(" Axis settings: " + axis.getName()));
		this.add(new Separator());

		ParallelChartContextMenuActionListener cmd = new ParallelChartContextMenuActionListener(mainWindow, chartFrame, axis);
		
		

		// invert axis
		JMenuItem invertAxisMenuItem = new JMenuItem("Invert Axis                                ");
		invertAxisMenuItem.setActionCommand("invertAxis");
		invertAxisMenuItem.addActionListener(cmd);
		//this.add(invertAxisMenuItem);
		
		// move column to left
		JMenuItem moveColumnLeftMenuItem = new JMenuItem("Move left                               ");
		this.addCustomHint(moveColumnLeftMenuItem, "(Left-click & drag)");
		moveColumnLeftMenuItem.setActionCommand("moveAxisLeft");
		moveColumnLeftMenuItem.addActionListener(cmd);
		this.add(moveColumnLeftMenuItem);

		// move column to right
		JMenuItem moveColumnRightMenuItem = new JMenuItem("Move right                               ");
		moveColumnRightMenuItem.setActionCommand("moveAxisRight");
		this.addCustomHint(moveColumnRightMenuItem, "(Left-click & drag)");
		moveColumnRightMenuItem.addActionListener(cmd);
		this.add(moveColumnRightMenuItem);

		// hide axis
		JMenuItem hideMenuItem = new JMenuItem("Hide axis                               ");
		hideMenuItem.setActionCommand("hideAxis");
		hideMenuItem.addActionListener(cmd);
		this.add(hideMenuItem);


		// increase distance
		JMenuItem increaseDistanceThisAxisMenuItem = new JMenuItem("Increase spacing                               ");
		increaseDistanceThisAxisMenuItem.setActionCommand("increaseDistanceThisAxis");
		this.addCustomHint(increaseDistanceThisAxisMenuItem, "(Ctrl+Mouse Whl Up)");
		increaseDistanceThisAxisMenuItem.addActionListener(cmd);
		this.add(increaseDistanceThisAxisMenuItem);

		// reduce distance
		JMenuItem reduceDistanceThisAxisMenuItem = new JMenuItem("Reduce spacing                                ");
		reduceDistanceThisAxisMenuItem.setActionCommand("reduceDistanceThisAxis");
		this.addCustomHint(reduceDistanceThisAxisMenuItem, "(Ctrl+Mouse Whl Down)");
		reduceDistanceThisAxisMenuItem.addActionListener(cmd);
		this.add(reduceDistanceThisAxisMenuItem);

		// all axes
		this.addSeparator();

		
		// increase distance
		JMenuItem increaseDistanceAllAxesMenuItem = new JMenuItem("Increase spacing all axes                               ");
		increaseDistanceAllAxesMenuItem.setActionCommand("increaseDistanceAllAxes");
		this.addCustomHint(increaseDistanceAllAxesMenuItem, "(Mouse Whl Up)");
		increaseDistanceAllAxesMenuItem.addActionListener(cmd);
		this.add(increaseDistanceAllAxesMenuItem);

		// reduce distance
		JMenuItem reduceDistanceAllAxesMenuItem = new JMenuItem("Reduce spacing all axes                                ");
		reduceDistanceAllAxesMenuItem.setActionCommand("reduceDistanceAllAxes");
		this.addCustomHint(reduceDistanceAllAxesMenuItem, "(Mouse Whl Down)");
		reduceDistanceAllAxesMenuItem.addActionListener(cmd);
		this.add(reduceDistanceAllAxesMenuItem);
                
                // categories
                this.addSeparator();
                
                ParallelSetsCategoriesMenu categoriesMenu = new ParallelSetsCategoriesMenu(mainWindow, chartFrame, axis);
                this.add(categoriesMenu);
                
                // parallel coordinates axis
                this.addSeparator();
                
                showCoordinatesAxisItem = new JCheckBoxMenuItem("Show coordinates axis");
                showCoordinatesAxisItem.setSelected(axis.isCoordinatesAxis());
                showCoordinatesAxisItem.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        showCoordinatesAxisItemAction();
                    }
                });
                if(axis.getParameter() != null && !axis.getParameter().isNumeric()) showCoordinatesAxisItem.setEnabled(false);
                this.add(showCoordinatesAxisItem);
                
                axisExpanded = new JCheckBoxMenuItem("Expand axis");
                axisExpanded.setSelected(axis.isExpanded());
                axisExpanded.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        axisExpanded();
                    }
                });
                //if(!axis.isExpanded()) axisExpanded.setEnabled(false);
                this.add(axisExpanded);
		
		

	}

	private void addCustomHint(JMenuItem item, String hint) {
		item.setLayout(new BorderLayout());
		JLabel label = new JLabel(hint + " ");
		label.setFont(new Font("SansSerif", Font.PLAIN, 12));
		label.setForeground(new Color(140, 140, 180));
		item.add(label, BorderLayout.EAST);
	}
        
        private void showCoordinatesAxisItemAction(){
            //axis.setCoordinatesAxis(showCoordinatesAxisItem.isSelected());
            chart.showCoordinatesOnAxis(axis, showCoordinatesAxisItem.isSelected());
            chartPanel.updateLayout();
            chartPanel.repaint();
        }
        
        private void axisExpanded(){
            axis.setExpanded(axisExpanded.isSelected());
            chartPanel.updateLayout();
            chartPanel.repaint();
        }

}
