
package org.xdat.gui.panels;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.DefaultListModel;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.xdat.Main;
import org.xdat.chart.*;
import org.xdat.gui.dialogs.CreateCustomAxisDialog;
import org.xdat.gui.frames.ChartFrame;


public class ParallelSetsChartSidebarPanel extends SidebarPanel {
    
    private class FilterMouseListener extends MouseAdapter {
        
        private int filterIndex = -1;
        
        @Override
        public void mousePressed(MouseEvent e) {
            filterIndex = getCategory(e.getX(), e.getY());

            if(e.getButton() == MouseEvent.BUTTON1) {
                dragStartY = e.getY();
                dragDeltaY = 0;

                ParallelSetsAxis axis = chart.getFilterAxis();
                AxisCategory cat = axis.getCategoryInDragZone(dragStartY);
                if(cat != null){
                    // drag category started
                    draggedCategory = cat;
                    setCursor(nResizeCursor);
                }
            }
        }
        
        @Override
        public void mouseReleased(MouseEvent e) {
            if(e.getButton() == MouseEvent.BUTTON1) {
                if(draggedCategory != null) {
                    setCursor(defaultCursor);
                    ParallelSetsFilterAxis axis = chart.getFilterAxis();
                    AxisCategory cat = axis.getCategoryInDragZone(e.getY());
                    if(cat != null && !cat.equals(draggedCategory)){
                        // drag category stop
                        int newIndex = axis.getCategories().indexOf(cat);
                        chart.moveCategory(draggedCategory, newIndex);
                        chart.setBuildAxisFromFiltersRequired(true);
                    } else if(filterIndex >= 0) {
                        // clicked on category
                        List<ParallelSetsFilter> filters = chart.getFilters();
                        if(filterIndex < filters.size()) {
                            chart.setSelectedFilter(filters.get(filterIndex));
                        }
                        else {
                            chart.setSelectedFilter(null);
                        }
                        //repaint();
                        //chartPanel.repaint();
                    }
                    draggedCategory = null;
                    chartPanel.updateLayout();
                    chartPanel.repaint();
                    repaint();
                }
            }
            else if(e.getButton() == MouseEvent.BUTTON3) {
                if(filterIndex >= 0) {
                    List<ParallelSetsFilter> filters = chart.getFilters();
                    if(filterIndex < filters.size()) {
                        colorButtonMouseReleased(e, filters.get(filterIndex));
                    }
                }
            }
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            if (draggedCategory != null){
                dragDeltaY = e.getY() - dragStartY;
                repaint();
            }
        }
    }
    
    class ResizeListener extends ComponentAdapter {
        @Override
        public void componentResized(ComponentEvent e) {
            repaint();
        }
    }

    static final Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
    static final Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
    static final Cursor nResizeCursor = new Cursor(Cursor.N_RESIZE_CURSOR);
    static final Cursor eResizeCursor = new Cursor(Cursor.E_RESIZE_CURSOR);

    private ParallelSetsChart chart;
    private ParallelSetsChartPanel chartPanel;
    private ChartFrame chartFrame;
    
    private DefaultListModel<ParallelSetsFilter> listModel;
    private JList<ParallelSetsFilter> filters;
    private JPanel filtersPanel;
    private JPanel listGridPanel;

    private JButton bCreateAxis;
    
    private List<JPanel> filterRows;
    private List<JLabel> filterLabels;

    private int dragStartY;
    private int dragDeltaY;
    private AxisCategory draggedCategory;

    public ParallelSetsChartSidebarPanel(Main mainWindow, ChartFrame chartFrame, ChartPanel chartPanel, ParallelSetsChart chart) {
        super(mainWindow, chartFrame, chartPanel, chart);
        FilterMouseListener filterMouseListener = new FilterMouseListener();
        addMouseListener(filterMouseListener);
        addMouseMotionListener(filterMouseListener);
        addComponentListener(new ResizeListener());
    }

    /**
     * Builds the panel.
     */
    @Override
    protected void buildPanel(JPanel parentPanel) {
        
        // create components
        chart = (ParallelSetsChart) getChart();
        chartPanel = (ParallelSetsChartPanel) getChartPanel();
        chartFrame = getChartFrame();

        parentPanel.setLayout(new BorderLayout());
        filtersPanel = new JPanel(new BorderLayout());
        if(chart.getLayoutMode() == ParallelSetsChart.TREE_LAYOUT) filtersPanel.setVisible(false);
        JPanel listPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
        listPanel.setPreferredSize(new Dimension(195, 140));
        
        JScrollPane listScrollPane = new JScrollPane();
        
        JPanel buttonsPanel = new JPanel(new BorderLayout());
        JPanel firstRow = new JPanel(new FlowLayout());
        JPanel secondRow = new JPanel(new FlowLayout());
        JButton buttonAdd = new JButton("Add new filter");
        JButton buttonRemove = new JButton("Remove selected filter");
        bCreateAxis = new JButton("Create axis from filters");
        buttonsPanel.add(firstRow, BorderLayout.NORTH);
        buttonsPanel.add(secondRow, BorderLayout.CENTER);
        firstRow.add(buttonAdd);
        firstRow.add(buttonRemove);
        secondRow.add(bCreateAxis);
        filtersPanel.add(buttonsPanel, BorderLayout.SOUTH);

        listModel = new DefaultListModel<>();
        filters = new JList<>(listModel);
        filters.setSelectedValue(chart.getSelectedFilter(), true);
        updateListModel();

        listPanel.add(filters);
        
        listGridPanel = new JPanel();
        listScrollPane.setViewportView(listGridPanel);
        initFilterList();

        JPanel treeButtonsPanel = new JPanel(new BorderLayout());
        if(chart.getLayoutMode() != ParallelSetsChart.TREE_LAYOUT) treeButtonsPanel.setVisible(false);
        JPanel treeButFirstRow = new JPanel(new FlowLayout());
        JButton bCreateCustomAxis = new JButton("Create custom axis");
        treeButFirstRow.add(bCreateCustomAxis);
        treeButtonsPanel.add(treeButFirstRow, BorderLayout.NORTH);

        switch (chart.getLayoutMode()){
            case ParallelSetsChart.TREE_LAYOUT:
                parentPanel.add(treeButtonsPanel, BorderLayout.NORTH);
                break;
            case ParallelSetsChart.BUNDLED_LAYOUT:
            case ParallelSetsChart.BUNDLED_COLOR_LAYOUT:
                parentPanel.add(filtersPanel, BorderLayout.NORTH);
        }

        buttonAdd.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                buttonAddActionPerformed(e);
            }
        });
        buttonRemove.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                buttonRemoveActionPerformed(e);
            }
        });
        bCreateAxis.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                bCreateAxisActionPerformed(e);
            }
        });
        filters.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                filtersValueChanged(e);
            }
        });
        bCreateCustomAxis.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                new CreateCustomAxisDialog(chartFrame, getMainWindow());
            }
        });

        setPreferredSize(new Dimension(300, 130));
        setMinimumSize(new Dimension(300, 130));

    }
    
    private void initFilterList(){
        listGridPanel.removeAll();
        listGridPanel.setLayout(new GridLayout(chart.getFilters().size(), 1));
        
        filterRows = new ArrayList<>();
        filterLabels = new ArrayList<>();
        
        for(int i = 0; i < chart.getFilters().size(); i++){
            ParallelSetsFilter filter = chart.getFilters().get(i);
            
            JPanel listRow = new JPanel(new BorderLayout());
            listRow.setBackground(Color.white);
            JPanel left = new JPanel(new FlowLayout());
            left.setOpaque(false);
            JPanel right = new JPanel(new FlowLayout());
            right.setOpaque(false);
            listRow.add(left, BorderLayout.WEST);
            listRow.add(right, BorderLayout.EAST);
            
            JCheckBox filterCheckbox = new JCheckBox();
            filterCheckbox.setOpaque(false);
            filterCheckbox.setSelected(filter.isActive());
            filterCheckbox.addChangeListener(new ChangeListener() {
                @Override
                public void stateChanged(ChangeEvent e) {
                    filterCheckboxChange(filterCheckbox, filter);
                }
            });
            //left.add(filterCheckbox);
            
            JLabel filterLabel = new JLabel(filter.toString());
            filterLabel.setOpaque(false);
            left.add(filterLabel);
            filterLabels.add(filterLabel);
            
            JPanel colorButBackground = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
            colorButBackground.setBackground(Color.white);
            colorButBackground.setOpaque(true);
            JPanel colorButton = new JPanel();
            colorButton.setBackground(filter.getColor());
            colorButton.addMouseListener(new MouseListener() {
                @Override
                public void mouseReleased(MouseEvent e) {
                    colorButtonMouseReleased(e, filter);
                }
                @Override
                public void mouseClicked(MouseEvent e) {}
                @Override
                public void mousePressed(MouseEvent e) {}
                @Override
                public void mouseEntered(MouseEvent e) {}
                @Override
                public void mouseExited(MouseEvent e) {}
            });
            colorButBackground.add(colorButton);
            right.add(colorButBackground);
            
            final int index = i;
            listRow.addMouseListener(new MouseListener() {
                @Override
                public void mouseReleased(MouseEvent e) {
                }
                @Override
                public void mouseClicked(MouseEvent e) {}
                @Override
                public void mousePressed(MouseEvent e) {
                    listRowMousePressed(index, filter);
                }
                @Override
                public void mouseEntered(MouseEvent e) {}
                @Override
                public void mouseExited(MouseEvent e) {}
            });
            
            listGridPanel.add(listRow);
            filterRows.add(listRow);
        }
        
        if(chart.getSelectedFilter() != null){
            int i = chart.getFilters().indexOf(chart.getSelectedFilter());
            filterRows.get(i).setBackground(Color.blue);
            filterLabels.get(i).setForeground(Color.white);
        }
        
        chartFrame.validate();
        chartFrame.repaint();
    }
    
    private void updateListModel(){
        listModel.clear();
        for (ParallelSetsFilter filter : chart.getFilters()) {
            listModel.addElement(filter);
        }
        
        filters.setSelectedValue(chart.getSelectedFilter(), true);
    }
    
    @Override
    public void updateData(){
        if(chart.getLayoutMode() == ParallelSetsChart.TREE_LAYOUT){
            filtersPanel.setVisible(false);
        }else{
            filtersPanel.setVisible(true);
            //updateListModel();
            initFilterList();
        }
    }
        
    /**
     * Is called before building the panel for the first time
     */
    @Override
    protected void initialize() {
    }
        
    private void colorButtonMouseReleased(MouseEvent e, ParallelSetsFilter filter){
        Color newColor = JColorChooser.showDialog(chartFrame, "Filter Color", filter.getColor());
        chart.changeFilterColor(filter, newColor);
        chartPanel.updateLayout();
        chartPanel.repaint();
        repaint();
    }
    
    private void filterCheckboxChange(JCheckBox checkbox, ParallelSetsFilter filter){
        filter.setActive(checkbox.isSelected());
        chart.buildDesignSets();
        chartPanel.updateLayout();
        chartPanel.repaint();
    }
    
    private void listRowMousePressed(int i, ParallelSetsFilter filter){
        if(chart.getSelectedFilter() != filter){
            for(int j = 0; j < chart.getFilters().size(); j++){
                //filterRows.get(j).setBackground(new Color(240,240,240));
                filterRows.get(j).setBackground(Color.white);
                filterLabels.get(j).setForeground(Color.black);
            }
            filterRows.get(i).setBackground(Color.blue);
            filterLabels.get(i).setForeground(Color.white);
            
            chart.setSelectedFilter(filter);
            chartPanel.repaint();
        }
    }
    
    private void buttonAddActionPerformed(ActionEvent e){
        chart.setSelectedFilter(null);
        
        repaint();
        chartPanel.repaint();
    }
    
    private void buttonRemoveActionPerformed(ActionEvent e){
        chart.removeFilter(chart.getSelectedFilter());
        chartPanel.updateLayout();
        repaint();
        chartPanel.repaint();
    }

    private void bCreateAxisActionPerformed(ActionEvent e){
        chart.copyFilterAxisToChart();
        chartPanel.updateLayout();
        repaint();
        chartPanel.repaint();
    }
    
    private void filtersValueChanged(ListSelectionEvent e){
        ParallelSetsFilter selectedValue = filters.getSelectedValue();
        if(selectedValue == null || selectedValue == chart.getSelectedFilter()) return;
        chart.setSelectedFilter(selectedValue);
        chartPanel.repaint();
    }

    @Override
    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        super.paint(g);
        if(chart.getLayoutMode() == ParallelSetsChart.TREE_LAYOUT) return;
        
        if(this.chart.isAntiAliasing()){
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        }
        
        chart.getFilterAxis().drawAxis(g2);

        if(draggedCategory != null){
            ParallelSetsAxis categoryAxis = chart.getFilterAxis();
            int[] pointsX = {
                categoryAxis.getPosition().x,
                categoryAxis.getPosition().x + categoryAxis.getBoxWidth(),
                categoryAxis.getPosition().x + categoryAxis.getBoxWidth(),
                categoryAxis.getPosition().x
            };
            int[] pointsY = {
                draggedCategory.getyUpPosition() + dragDeltaY,
                draggedCategory.getyUpPosition() + dragDeltaY,
                draggedCategory.getyUpPosition() + draggedCategory.getHeight() + dragDeltaY,
                draggedCategory.getyUpPosition() + draggedCategory.getHeight() + dragDeltaY
            };
            g.setColor(new Color(100, 100, 100, 125));
            g.fillPolygon(pointsX, pointsY, 4);
        }
    }
    
    public int getCategory(int x, int y) {
            ParallelSetsFilterAxis axis = chart.getFilterAxis();
            int i = 0;
            if(axis.containsPoint(x, y)) {
                for(AxisCategory c : axis.getCategories()) {
                    if(c.containsPoint(y)) {
                        return i;
                    }
                    i++;
                }
            }
            else {
                return -1;
            }
            return i;
        }
    
    

}
