
package org.xdat.workerThreads;

import javax.swing.JOptionPane;
import javax.swing.ProgressMonitor;
import javax.swing.SwingWorker;

import org.xdat.Main;
import org.xdat.chart.ParallelSetsChart;
import org.xdat.exceptions.NoParametersDefinedException;
import org.xdat.gui.frames.ChartFrame;
import org.xdat.gui.tables.DataSheetTableColumnModel;


public class ParallelSetsChartCreationThread extends SwingWorker {
	static final boolean printLog = false;

	private Main mainWindow;

	private ProgressMonitor progressMonitor;

	public ParallelSetsChartCreationThread(Main mainWindow, ProgressMonitor progressMonitor) {
		this.mainWindow = mainWindow;
		this.progressMonitor = progressMonitor;
	}

	@Override
	public Object doInBackground() {
		// log("do in background invoked from Thread "+Thread.currentThread().getId());
		// log("Create: Creation invoked at "+(new Date()).toString());
		DataSheetTableColumnModel cm = (DataSheetTableColumnModel) this.mainWindow.getDataSheetTablePanel().getDataTable().getColumnModel();
		ParallelSetsChart chart = new ParallelSetsChart(mainWindow.getDataSheet(), cm, progressMonitor, mainWindow.getUniqueChartId(ParallelSetsChart.class));

		// log("doInBackground: progressMonitor isCanceled = "+progressMonitor.isCanceled());
		if (progressMonitor.isCanceled()) {
			// log("doInBackground: progressMonitor is Canceled ");
			return null;
		} else {
			try {
				new ChartFrame(mainWindow, chart);
				this.mainWindow.getCurrentSession().addChart(chart);
			} catch (NoParametersDefinedException e) {
				JOptionPane.showMessageDialog(mainWindow, "Cannot create chart when no parameters are defined.", "No parameters defined!", JOptionPane.ERROR_MESSAGE);
			}
			return null;
		}
	}

	@Override
	protected void done() {
		super.done();
		log("done: progressMonitor isCanceled = " + progressMonitor.isCanceled());
	}

	private void log(String message) {
		if (ParallelSetsChartCreationThread.printLog && Main.isLoggingEnabled()) {
			System.out.println(this.getClass().getName() + "." + message);
		}
	}
}

